@@ -1,18 +1,7 @@
 /*
- * Copyright 2017 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2017 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "audiowrapper.h"
@@ -22,7 +11,6 @@
 #include "vlcLogging.h"
 #include "powermanagementinterface.h"
 
-#include <QTimer>
 #include <QAudio>
 #include <QDir>
 
@@ -138,6 +126,10 @@ AudioWrapper::AudioWrapper(QObject *parent) : QObject(parent), d(std::make_uniqu
 AudioWrapper::~AudioWrapper()
 {
     if (d->mInstance) {
+        d->mPowerInterface.setPreventSleep(false);
+        if (d->mPlayer && d->mPreviousPlayerState != QMediaPlayer::StoppedState) {
+            libvlc_media_player_stop(d->mPlayer);
+        }
         libvlc_release(d->mInstance);
     }
 }
@@ -260,6 +252,8 @@ void AudioWrapper::setSource(const QUrl &source)
     d->signalMediaStatusChange(QMediaPlayer::LoadingMedia);
     d->signalMediaStatusChange(QMediaPlayer::LoadedMedia);
     d->signalMediaStatusChange(QMediaPlayer::BufferedMedia);
+
+    d->mHasSavedPosition = false;
 }
 
 void AudioWrapper::setPosition(qint64 position)
