@@ -1,18 +1,7 @@
 /*
- * Copyright 2016-2017 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2016 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #ifndef DATABASEINTERFACE_H
@@ -27,16 +16,13 @@
 #include <QString>
 #include <QHash>
 #include <QList>
-#include <QVariant>
 #include <QUrl>
 #include <QDateTime>
-#include <QPair>
 
 #include <memory>
 #include <optional>
 
 class DatabaseInterfacePrivate;
-class QMutex;
 class QSqlRecord;
 class QSqlQuery;
 
@@ -46,6 +32,16 @@ class ELISALIB_EXPORT DatabaseInterface : public QObject
 
 public:
 
+    enum DatabaseVersion {
+        V9 = 9,
+        V11 = 11,
+        V12 = 12,
+        V13 = 13,
+        V14 = 14,
+        V15 = 15,
+        V16 = 16, //Does not exist yet, for testing purpose only.
+    };
+
     explicit DatabaseInterface(QObject *parent = nullptr);
 
     ~DatabaseInterface() override;
@@ -76,6 +72,10 @@ public:
 
     DataTypes::ListArtistDataType allArtistsDataByGenre(const QString &genre);
 
+    DataTypes::ArtistDataType artistDataFromDatabaseId(qulonglong id);
+
+    qulonglong artistIdFromName(const QString &name);
+
     DataTypes::ListGenreDataType allGenresData();
 
     bool internalArtistMatchGenre(qulonglong databaseId, const QString &genre);
@@ -152,8 +152,6 @@ public Q_SLOTS:
 
     void clearData();
 
-    void insertRadio(const DataTypes::TrackDataType &oneTrack);
-
     void removeRadio(qulonglong radioId);
 
 private:
@@ -169,11 +167,11 @@ private:
 
     void recordModifiedAlbum(qulonglong albumId);
 
-    bool startTransaction() const;
+    bool startTransaction();
 
-    bool finishTransaction() const;
+    bool finishTransaction();
 
-    bool rollBackTransaction() const;
+    bool rollBackTransaction();
 
     QList<qulonglong> fetchTrackIds(qulonglong albumId);
 
@@ -233,11 +231,9 @@ private:
     qulonglong internalInsertTrack(const DataTypes::TrackDataType &oneModifiedTrack,
                                    const QHash<QString, QUrl> &covers, bool &isInserted);
 
-    DataTypes::TrackDataType buildTrackFromDatabaseRecord(const QSqlRecord &trackRecord) const;
+    [[nodiscard]] DataTypes::TrackDataType buildTrackDataFromDatabaseRecord(const QSqlRecord &trackRecord) const;
 
-    DataTypes::TrackDataType buildTrackDataFromDatabaseRecord(const QSqlRecord &trackRecord) const;
-
-    DataTypes::TrackDataType buildRadioDataFromDatabaseRecord(const QSqlRecord &trackRecord) const;
+    [[nodiscard]] DataTypes::TrackDataType buildRadioDataFromDatabaseRecord(const QSqlRecord &trackRecord) const;
 
     void internalRemoveTracksList(const QList<QUrl> &removedTracks);
 
@@ -261,6 +257,8 @@ private:
 
     DataTypes::AlbumDataType internalOneAlbumPartialData(qulonglong databaseId);
 
+    DataTypes::ArtistDataType internalOneArtistPartialData(qulonglong databaseId);
+
     DataTypes::ListTrackDataType internalAllTracksPartialData();
 
     DataTypes::ListRadioDataType internalAllRadiosPartialData();
@@ -288,6 +286,8 @@ private:
     void updateAlbumArtist(qulonglong albumId, const QString &title, const QString &albumPath,
                            const QString &artistName);
 
+    bool updateAlbumCover(qulonglong albumId, const QUrl &albumArtUri);
+
     void updateTrackStatistics(const QUrl &fileName, const QDateTime &time);
 
     void createDatabaseV9();
@@ -302,6 +302,10 @@ private:
 
     void upgradeDatabaseV14();
 
+    void upgradeDatabaseV15();
+
+    void upgradeDatabaseV16();
+
     void checkDatabaseSchema();
 
     void checkAlbumsTableSchema();
@@ -322,6 +326,22 @@ private:
 
     void resetDatabase();
 
+    void manageNewDatabaseVersion();
+
+    void dropTable(const QString &table);
+
+    void setDatabaseVersionInTable(int version);
+
+    void createDatabaseVersionTable();
+
+    void manageNewDatabaseVersionInitRequests();
+
+    void callUpgradeFunctionForVersion(DatabaseVersion databaseVersion);
+
+    void internalInsertOneTrack(const DataTypes::TrackDataType &oneTrack, const QHash<QString, QUrl> &covers);
+
+    void internalInsertOneRadio(const DataTypes::TrackDataType &oneTrack);
+
     std::unique_ptr<DatabaseInterfacePrivate> d;
 
 };
