@@ -1,19 +1,8 @@
 /*
- * Copyright 2017 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- * Copyright (C) 2012 Aleix Pol Gonzalez <aleixpol@blue-systems.com>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2017 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+   SPDX-FileCopyrightText: 2012 (c) Aleix Pol Gonzalez <aleixpol@blue-systems.com>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "elisaapplication.h"
@@ -21,6 +10,7 @@
 #include "musiclistenersmanager.h"
 
 #include "mediaplaylist.h"
+#include "mediaplaylistproxymodel.h"
 #include "audiowrapper.h"
 #include "manageaudioplayer.h"
 #include "managemediaplayercontrol.h"
@@ -29,6 +19,7 @@
 
 #include "elisa_settings.h"
 #include <KConfigCore/KAuthorized>
+#include <KLocalizedString>
 
 #if defined KF5ConfigWidgets_FOUND && KF5ConfigWidgets_FOUND
 #include <KConfigWidgets/KStandardAction>
@@ -42,14 +33,13 @@
 #include <KXmlGui/KShortcutsDialog>
 #endif
 
-#if defined KF5KCMUtils_FOUND && KF5KCMUtils_FOUND
-#include <KCMUtils/KCMultiDialog>
-#endif
-
+#include <KColorSchemeManager>
 #include <KCoreAddons/KAboutData>
+#include <KIO/OpenFileManagerWindowJob>
 
-#include <QQmlApplicationEngine>
-#include <QCoreApplication>
+#include <QQmlEngine>
+#include <QQmlComponent>
+#include <QGuiApplication>
 #include <QDesktopServices>
 #include <QDialog>
 #include <QPointer>
@@ -58,8 +48,10 @@
 #include <QUrl>
 #include <QFileInfo>
 #include <QDir>
+#include <QFileSystemWatcher>
 #include <QKeyEvent>
 #include <QDebug>
+#include <QFileSystemWatcher>
 
 #include <memory>
 
@@ -73,24 +65,20 @@ public:
     #endif
     {
         Q_UNUSED(parent)
-
-        auto configurationFileName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
-        configurationFileName += QStringLiteral("/elisarc");
-        Elisa::ElisaConfiguration::instance(configurationFileName);
-        Elisa::ElisaConfiguration::self()->load();
-        Elisa::ElisaConfiguration::self()->save();
     }
 
 #if defined KF5XmlGui_FOUND && KF5XmlGui_FOUND
     KActionCollection mCollection;
 #endif
 
-    ElisaUtils::EntryDataList mArguments;
+    DataTypes::EntryDataList mArguments;
 
     std::unique_ptr<MusicListenersManager> mMusicManager;
 
     std::unique_ptr<MediaPlayList> mMediaPlayList;
 
+    std::unique_ptr<MediaPlayListProxyModel> mMediaPlayListProxyModel;
+
     std::unique_ptr<AudioWrapper> mAudioWrapper;
 
     std::unique_ptr<ManageAudioPlayer> mAudioControl;
@@ -99,17 +87,48 @@ public:
 
     std::unique_ptr<ManageHeaderBar> mManageHeaderBar;
 
-    QQmlApplicationEngine *mEngine = nullptr;
+    QQmlEngine *mEngine = nullptr;
 
+    QFileSystemWatcher mConfigFileWatcher;
+
+    KColorSchemeManager *mSchemes;
 };
 
 ElisaApplication::ElisaApplication(QObject *parent) : QObject(parent), d(std::make_unique<ElisaApplicationPrivate>(this))
 {
+    auto configurationFileName = QStandardPaths::writableLocation(QStandardPaths::ConfigLocation);
+    configurationFileName += QStringLiteral("/elisarc");
+    Elisa::ElisaConfiguration::instance(configurationFileName);
+    Elisa::ElisaConfiguration::self()->load();
+    Elisa::ElisaConfiguration::self()->save();
+
+    d->mSchemes = new KColorSchemeManager(this);
+
+    d->mConfigFileWatcher.addPath(Elisa::ElisaConfiguration::self()->config()->name());
+
+    connect(Elisa::ElisaConfiguration::self(), &Elisa::ElisaConfiguration::configChanged,
+            this, &ElisaApplication::configChanged);
+
+    connect(&d->mConfigFileWatcher, &QFileSystemWatcher::fileChanged,
+            this, &ElisaApplication::configChanged);
+
+    connect(static_cast<QGuiApplication*>(QCoreApplication::instance()), &QGuiApplication::commitDataRequest,
+            this, &ElisaApplication::commitDataRequest);
 }
 
 ElisaApplication::~ElisaApplication()
 = default;
 
+QAbstractItemModel *ElisaApplication::colorSchemesModel()
+{
+    return d->mSchemes->model();
+}
+
+void ElisaApplication::activateColorScheme(const QString &name)
+{
+    d->mSchemes->activateScheme(d->mSchemes->indexForScheme(name));
+}
+
 void ElisaApplication::setupActions(const QString &actionName)
 {
 #if defined KF5XmlGui_FOUND && KF5XmlGui_FOUND
@@ -151,21 +170,36 @@ void ElisaApplication::setupActions(const QString &actionName)
     if (actionName == QLatin1String("toggle_playlist") && KAuthorized::authorizeAction(actionName)) {
         auto togglePlaylistAction = d->mCollection.addAction(actionName, this, &ElisaApplication::togglePlaylist);
         togglePlaylistAction->setShortcut(QKeySequence(Qt::Key_F9));
-        togglePlaylistAction->setText(QStringLiteral("Toggle Playlist"));
+        togglePlaylistAction->setText(i18n("Toggle Playlist"));
     }
 
     if (actionName == QLatin1String("Seek") && KAuthorized::authorizeAction(actionName)) {
             auto seekAction = d->mCollection.addAction(actionName, this, &ElisaApplication::seek);
+            seekAction->setText(i18n("Seek forward 10 seconds"));
             d->mCollection.setDefaultShortcut(seekAction, QKeySequence(Qt::SHIFT + Qt::Key_Right));
     }
 
     if (actionName == QLatin1String("Scrub") && KAuthorized::authorizeAction(actionName)) {
             auto scrubAction = d->mCollection.addAction(actionName, this, &ElisaApplication::scrub);
+            scrubAction->setText(i18n("Scrub backwards 10 seconds"));
             d->mCollection.setDefaultShortcut(scrubAction, QKeySequence(Qt::SHIFT + Qt::Key_Left));
     }
 
+    if (actionName == QLatin1String("NextTrack") && KAuthorized::authorizeAction(actionName)) {
+            auto nextTrackAction = d->mCollection.addAction(actionName, this, &ElisaApplication::nextTrack);
+            nextTrackAction->setText(i18n("Go to next track"));
+            d->mCollection.setDefaultShortcut(nextTrackAction, QKeySequence(Qt::CTRL + Qt::Key_Left));
+    }
+
+    if (actionName == QLatin1String("PreviousTrack") && KAuthorized::authorizeAction(actionName)) {
+            auto previousTrackAction = d->mCollection.addAction(actionName, this, &ElisaApplication::previousTrack);
+            previousTrackAction->setText(i18n("Go to previous track"));
+            d->mCollection.setDefaultShortcut(previousTrackAction, QKeySequence(Qt::CTRL + Qt::Key_Right));
+    }
+
     if (actionName == QLatin1String("Play-Pause") && KAuthorized::authorizeAction(actionName)) {
             auto playPauseAction = d->mCollection.addAction(actionName, this, &ElisaApplication::playPause);
+            playPauseAction->setText(i18n("Play/pause"));
             d->mCollection.setDefaultShortcut(playPauseAction, QKeySequence(Qt::Key_Space));
     }
 
@@ -178,7 +212,7 @@ void ElisaApplication::setupActions(const QString &actionName)
 #endif
 }
 
-void ElisaApplication::setArguments(const ElisaUtils::EntryDataList &newArguments)
+void ElisaApplication::setArguments(const DataTypes::EntryDataList &newArguments)
 {
     if (d->mArguments == newArguments) {
         return;
@@ -188,7 +222,7 @@ void ElisaApplication::setArguments(const ElisaUtils::EntryDataList &newArgument
     Q_EMIT argumentsChanged();
 
     if (!d->mArguments.isEmpty()) {
-        Q_EMIT enqueue(d->mArguments, ElisaUtils::FileName,
+        Q_EMIT enqueue(d->mArguments,
                        ElisaUtils::PlayListEnqueueMode::AppendPlayList,
                        ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
@@ -203,7 +237,7 @@ void ElisaApplication::activateActionRequested(const QString &actionName, const
 void ElisaApplication::activateRequested(const QStringList &arguments, const QString &workingDirectory)
 {
     if (arguments.size() > 1) {
-        auto realArguments = ElisaUtils::EntryDataList{};
+        auto realArguments = DataTypes::EntryDataList{};
 
         bool isFirst = true;
         for (const auto &oneArgument : arguments) {
@@ -212,11 +246,11 @@ void ElisaApplication::activateRequested(const QStringList &arguments, const QSt
                 continue;
             }
 
-            realArguments.push_back(ElisaUtils::EntryData{{}, {}, QUrl(oneArgument)});
+            realArguments.push_back({{{DataTypes::ElementTypeRole, ElisaUtils::Track},
+                                      {DataTypes::ResourceRole, QUrl::fromUserInput(oneArgument)}}, {}, {}});
         }
 
         Q_EMIT enqueue(checkFileListAndMakeAbsolute(realArguments, workingDirectory),
-                       ElisaUtils::FileName,
                        ElisaUtils::PlayListEnqueueMode::AppendPlayList,
                        ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
@@ -259,7 +293,7 @@ void ElisaApplication::reportBug()
 void ElisaApplication::configureShortcuts()
 {
 #if defined KF5XmlGui_FOUND && KF5XmlGui_FOUND
-    KShortcutsDialog dlg(KShortcutsEditor::AllActions, KShortcutsEditor::LetterShortcutsAllowed, nullptr);
+    KShortcutsDialog dlg(KShortcutsEditor::ApplicationAction, KShortcutsEditor::LetterShortcutsAllowed, nullptr);
     dlg.setModal(true);
     dlg.addCollection(&d->mCollection);
     dlg.configure();
@@ -273,7 +307,10 @@ void ElisaApplication::configureElisa()
         return;
     }
 
-    d->mEngine->load(QUrl(QStringLiteral("qrc:/qml/ElisaConfigurationDialog.qml")));
+    QQmlComponent configurationDialogComponent(d->mEngine,
+                                               QUrl(QStringLiteral("qrc:/qml/ElisaConfigurationDialog.qml")));
+
+    configurationDialogComponent.create();
 }
 
 void ElisaApplication::goBack() {}
@@ -286,33 +323,54 @@ void ElisaApplication::seek() {}
 
 void ElisaApplication::scrub() {}
 
+void ElisaApplication::nextTrack() {}
+
+void ElisaApplication::previousTrack() {}
+
 void ElisaApplication::playPause() {}
 
-ElisaUtils::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const ElisaUtils::EntryDataList &filesList,
+void ElisaApplication::configChanged()
+{
+    auto currentConfiguration = Elisa::ElisaConfiguration::self();
+
+    d->mConfigFileWatcher.addPath(currentConfiguration->config()->name());
+
+    currentConfiguration->load();
+    currentConfiguration->read();
+
+    Q_EMIT showProgressOnTaskBarChanged();
+    Q_EMIT showSystemTrayIconChanged();
+    Q_EMIT embeddedViewChanged();
+}
+
+DataTypes::EntryDataList ElisaApplication::checkFileListAndMakeAbsolute(const DataTypes::EntryDataList &filesList,
                                                                          const QString &workingDirectory) const
 {
-    auto filesToOpen = ElisaUtils::EntryDataList{};
+    auto filesToOpen = DataTypes::EntryDataList{};
 
     for (const auto &oneFile : filesList) {
-        if (std::get<2>(oneFile).scheme().isEmpty() || std::get<2>(oneFile).isLocalFile()) {
-            auto newFile = QFileInfo(std::get<2>(oneFile).toLocalFile());
-            if (std::get<2>(oneFile).scheme().isEmpty()) {
-                newFile = QFileInfo(std::get<2>(oneFile).toString());
+        auto trackUrl = std::get<0>(oneFile)[DataTypes::ResourceRole].toUrl();
+        if (trackUrl.scheme().isEmpty() || trackUrl.isLocalFile()) {
+            auto newFile = QFileInfo(trackUrl.toLocalFile());
+            if (trackUrl.scheme().isEmpty()) {
+                newFile = QFileInfo(trackUrl.toString());
             }
 
             if (newFile.isRelative()) {
-                if (std::get<2>(oneFile).scheme().isEmpty()) {
-                    newFile.setFile(workingDirectory, std::get<2>(oneFile).toString());
+                if (trackUrl.scheme().isEmpty()) {
+                    newFile.setFile(workingDirectory, trackUrl.toString());
                 } else {
-                    newFile.setFile(workingDirectory, std::get<2>(oneFile).toLocalFile());
+                    newFile.setFile(workingDirectory, trackUrl.toLocalFile());
                 }
             }
 
             if (newFile.exists()) {
-                filesToOpen.push_back(ElisaUtils::EntryData{{}, {}, QUrl::fromLocalFile(newFile.canonicalFilePath())});
+                auto trackData = std::get<0>(oneFile);
+                trackData[DataTypes::ResourceRole] = QUrl::fromLocalFile(newFile.canonicalFilePath());
+                filesToOpen.push_back({trackData, std::get<1>(oneFile), {}});
             }
         } else {
-            filesToOpen.push_back(ElisaUtils::EntryData{{}, {}, std::get<2>(oneFile)});
+            filesToOpen.push_back(oneFile);
         }
     }
 
@@ -327,7 +385,7 @@ void ElisaApplication::initialize()
     Q_EMIT initializationDone();
 }
 
-void ElisaApplication::setQmlEngine(QQmlApplicationEngine *engine)
+void ElisaApplication::setQmlEngine(QQmlEngine *engine)
 {
     d->mEngine = engine;
 }
@@ -338,16 +396,19 @@ void ElisaApplication::initializeModels()
     Q_EMIT musicManagerChanged();
 
     d->mMediaPlayList = std::make_unique<MediaPlayList>();
+    d->mMusicManager->subscribeForTracks(d->mMediaPlayList.get());
     Q_EMIT mediaPlayListChanged();
 
+    d->mMediaPlayListProxyModel = std::make_unique<MediaPlayListProxyModel>();
+    d->mMediaPlayListProxyModel->setPlayListModel(d->mMediaPlayList.get());
+    Q_EMIT mediaPlayListProxyModelChanged();
+
     d->mMusicManager->setElisaApplication(this);
 
-    d->mMediaPlayList->setMusicListenersManager(d->mMusicManager.get());
     QObject::connect(this, &ElisaApplication::enqueue,
-                     d->mMediaPlayList.get(), static_cast<void (MediaPlayList::*)(const ElisaUtils::EntryDataList&,
-                                                                                  ElisaUtils::PlayListEntryType,
-                                                                                  ElisaUtils::PlayListEnqueueMode,
-                                                                                  ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayList::enqueue));
+                     d->mMediaPlayListProxyModel.get(), static_cast<void (MediaPlayListProxyModel::*)(const DataTypes::EntryDataList&,
+                                                                                            ElisaUtils::PlayListEnqueueMode,
+                                                                                            ElisaUtils::PlayListEnqueueTriggerPlay)>(&MediaPlayListProxyModel::enqueue));
 }
 
 void ElisaApplication::initializePlayer()
@@ -366,7 +427,7 @@ void ElisaApplication::initializePlayer()
     d->mAudioControl->setTitleRole(MediaPlayList::TitleRole);
     d->mAudioControl->setUrlRole(MediaPlayList::ResourceRole);
     d->mAudioControl->setIsPlayingRole(MediaPlayList::IsPlayingRole);
-    d->mAudioControl->setPlayListModel(d->mMediaPlayList.get());
+    d->mAudioControl->setPlayListModel(d->mMediaPlayListProxyModel.get());
 
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::playerPlay, d->mAudioWrapper.get(), &AudioWrapper::play);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::playerPause, d->mAudioWrapper.get(), &AudioWrapper::pause);
@@ -374,19 +435,22 @@ void ElisaApplication::initializePlayer()
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::seek, d->mAudioWrapper.get(), &AudioWrapper::seek);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::saveUndoPositionInAudioWrapper, d->mAudioWrapper.get(), &AudioWrapper::saveUndoPosition);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::restoreUndoPositionInAudioWrapper, d->mAudioWrapper.get(), &AudioWrapper::restoreUndoPosition);
-    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::skipNextTrack, d->mMediaPlayList.get(), &MediaPlayList::skipNextTrack);
-    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::sourceInError, d->mMediaPlayList.get(), &MediaPlayList::trackInError);
+
+    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::skipNextTrack, d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::skipNextTrack);
+    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::sourceInError, d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::trackInError);
+
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::sourceInError, d->mMusicManager.get(), &MusicListenersManager::playBackError);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::playerSourceChanged, d->mAudioWrapper.get(), &AudioWrapper::setSource);
     QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::startedPlayingTrack,
                      d->mMusicManager->viewDatabase(), &DatabaseInterface::trackHasStartedPlaying);
-    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::currentPlayingForRadiosChanged, d->mMediaPlayList.get(), &MediaPlayList::updateRadioData);
+    QObject::connect(d->mAudioControl.get(), &ManageAudioPlayer::updateData, d->mMediaPlayList.get(), &MediaPlayList::setData);
 
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::ensurePlay, d->mAudioControl.get(), &ManageAudioPlayer::ensurePlay);
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::playListFinished, d->mAudioControl.get(), &ManageAudioPlayer::playListFinished);
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::currentTrackChanged, d->mAudioControl.get(), &ManageAudioPlayer::setCurrentTrack);
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::clearPlayListPlayer, d->mAudioControl.get(), &ManageAudioPlayer::saveForUndoClearPlaylist);
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::undoClearPlayListPlayer, d->mAudioControl.get(), &ManageAudioPlayer::restoreForUndoClearPlaylist);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::ensurePlay, d->mAudioControl.get(), &ManageAudioPlayer::ensurePlay);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::playListFinished, d->mAudioControl.get(), &ManageAudioPlayer::playListFinished);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::currentTrackChanged, d->mAudioControl.get(), &ManageAudioPlayer::setCurrentTrack);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::clearPlayListPlayer, d->mAudioControl.get(), &ManageAudioPlayer::saveForUndoClearPlaylist);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::undoClearPlayListPlayer, d->mAudioControl.get(), &ManageAudioPlayer::restoreForUndoClearPlaylist);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::seek, d->mAudioWrapper.get(), &AudioWrapper::seek);
 
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::playbackStateChanged,
                      d->mAudioControl.get(), &ManageAudioPlayer::setPlayerPlaybackState);
@@ -397,9 +461,9 @@ void ElisaApplication::initializePlayer()
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::positionChanged, d->mAudioControl.get(), &ManageAudioPlayer::setPlayerPosition);
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::currentPlayingForRadiosChanged, d->mAudioControl.get(), &ManageAudioPlayer::setCurrentPlayingForRadios);
 
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::currentTrackChanged, d->mPlayerControl.get(), &ManageMediaPlayerControl::setCurrentTrack);
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::previousTrackChanged, d->mPlayerControl.get(), &ManageMediaPlayerControl::setPreviousTrack);
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::nextTrackChanged, d->mPlayerControl.get(), &ManageMediaPlayerControl::setNextTrack);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::currentTrackChanged, d->mPlayerControl.get(), &ManageMediaPlayerControl::setCurrentTrack);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::previousTrackChanged, d->mPlayerControl.get(), &ManageMediaPlayerControl::setPreviousTrack);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::nextTrackChanged, d->mPlayerControl.get(), &ManageMediaPlayerControl::setNextTrack);
 
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::playing, d->mPlayerControl.get(), &ManageMediaPlayerControl::playerPlaying);
     QObject::connect(d->mAudioWrapper.get(), &AudioWrapper::paused, d->mPlayerControl.get(), &ManageMediaPlayerControl::playerPausedOrStopped);
@@ -415,10 +479,11 @@ void ElisaApplication::initializePlayer()
     d->mManageHeaderBar->setTrackTypeRole(MediaPlayList::ElementTypeRole);
     d->mManageHeaderBar->setAlbumIdRole(MediaPlayList::AlbumIdRole);
     d->mManageHeaderBar->setIsValidRole(MediaPlayList::IsValidRole);
-    QObject::connect(d->mMediaPlayList.get(), &MediaPlayList::currentTrackChanged, d->mManageHeaderBar.get(), &ManageHeaderBar::setCurrentTrack);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::currentTrackChanged, d->mManageHeaderBar.get(), &ManageHeaderBar::setCurrentTrack);
+    QObject::connect(d->mMediaPlayListProxyModel.get(), &MediaPlayListProxyModel::currentTrackDataChanged, d->mManageHeaderBar.get(), &ManageHeaderBar::updateCurrentTrackData);
 
     if (!d->mArguments.isEmpty()) {
-        Q_EMIT enqueue(d->mArguments, ElisaUtils::FileName,
+        Q_EMIT enqueue(d->mArguments,
                        ElisaUtils::PlayListEnqueueMode::AppendPlayList,
                        ElisaUtils::PlayListEnqueueTriggerPlay::TriggerPlay);
     }
@@ -458,10 +523,10 @@ void ElisaApplication::installKeyEventFilter(QObject *object)
 
 bool ElisaApplication::eventFilter(QObject *object, QEvent *event)
 {
-    Q_UNUSED(object);
+    Q_UNUSED(object)
 
-    QKeyEvent *keyEvent = static_cast<QKeyEvent*>(event);
-    auto playPauseAction = d->mCollection.action(tr("Play-Pause"));
+    auto *keyEvent = static_cast<QKeyEvent*>(event);
+    auto playPauseAction = d->mCollection.action(QStringLiteral("Play-Pause"));
     if (keyEvent->key() == Qt::Key_Space && playPauseAction->shortcut()[0] == Qt::Key_Space) {
         return true;
     }
@@ -469,7 +534,12 @@ bool ElisaApplication::eventFilter(QObject *object, QEvent *event)
     return false;
 }
 
-const ElisaUtils::EntryDataList &ElisaApplication::arguments() const
+void ElisaApplication::showInFolder(QUrl filePath)
+{
+    KIO::highlightInFileManager({filePath});
+}
+
+const DataTypes::EntryDataList &ElisaApplication::arguments() const
 {
     return d->mArguments;
 }
@@ -484,6 +554,12 @@ MediaPlayList *ElisaApplication::mediaPlayList() const
     return d->mMediaPlayList.get();
 }
 
+MediaPlayListProxyModel *ElisaApplication::mediaPlayListProxyModel() const
+{
+    return d->mMediaPlayListProxyModel.get();
+}
+
+
 AudioWrapper *ElisaApplication::audioPlayer() const
 {
     return d->mAudioWrapper.get();
@@ -504,4 +580,48 @@ ManageHeaderBar *ElisaApplication::manageHeaderBar() const
     return d->mManageHeaderBar.get();
 }
 
+bool ElisaApplication::showProgressOnTaskBar() const
+{
+    auto currentConfiguration = Elisa::ElisaConfiguration::self();
+
+    return currentConfiguration->showProgressOnTaskBar();
+}
+
+bool ElisaApplication::showSystemTrayIcon() const
+{
+    auto currentConfiguration = Elisa::ElisaConfiguration::self();
+
+    return currentConfiguration->showSystemTrayIcon();
+}
+
+ElisaUtils::PlayListEntryType ElisaApplication::embeddedView() const
+{
+    ElisaUtils::PlayListEntryType result = ElisaUtils::Unknown;
+
+    switch (Elisa::ElisaConfiguration::self()->embeddedView())
+    {
+    case 0:
+        result = ElisaUtils::Unknown;
+        break;
+    case 1:
+        result = ElisaUtils::Album;
+        break;
+    case 2:
+        result = ElisaUtils::Artist;
+        break;
+    case 3:
+        result = ElisaUtils::Genre;
+        break;
+    }
+
+    return result;
+}
+
+int ElisaApplication::initialViewIndex() const
+{
+    int result = Elisa::ElisaConfiguration::initialView();
+
+    return result;
+}
+
 #include "moc_elisaapplication.cpp"
