@@ -44,43 +44,73 @@ class ELISALIB_EXPORT ElisaConfigurationDialog : public QObject
                WRITE setEmbeddedView
                NOTIFY embeddedViewChanged)
 
+    Q_PROPERTY(int initialViewIndex
+               READ initialViewIndex
+               WRITE setInitialViewIndex
+               NOTIFY initialViewIndexChanged)
+
     Q_PROPERTY(bool isDirty
                READ isDirty
                NOTIFY isDirtyChanged)
 
+    Q_PROPERTY(bool playAtStartup
+               READ playAtStartup
+               WRITE setPlayAtStartup
+               NOTIFY playAtStartupChanged)
+
+        Q_PROPERTY(QString colorScheme
+                   READ colorScheme
+                   WRITE setColorScheme
+                   NOTIFY colorSchemeChanged)
+
 public:
 
     explicit ElisaConfigurationDialog(QObject *parent = nullptr);
 
     ~ElisaConfigurationDialog() override;
 
-    QStringList rootPath() const;
+    [[nodiscard]] QStringList rootPath() const;
 
-    bool isDirty() const
+    [[nodiscard]] bool isDirty() const
     {
         return mIsDirty;
     }
 
-    bool showProgressInTaskBar() const
+    [[nodiscard]] bool showProgressInTaskBar() const
     {
         return mShowProgressInTaskBar;
     }
 
-    bool showSystemTrayIcon() const
+    [[nodiscard]] bool showSystemTrayIcon() const
     {
         return mShowSystemTrayIcon;
     }
 
-    bool forceUsageOfFastFileSearch() const
+    [[nodiscard]] bool forceUsageOfFastFileSearch() const
     {
         return mForceUsageOfFastFileSearch;
     }
 
-    ElisaUtils::PlayListEntryType embeddedView() const
+    [[nodiscard]] ElisaUtils::PlayListEntryType embeddedView() const
     {
         return mEmbeddedView;
     }
 
+    [[nodiscard]] int initialViewIndex() const
+    {
+        return mInitialViewIndex;
+    }
+
+    [[nodiscard]] bool playAtStartup() const
+    {
+        return mPlayAtStartup;
+    }
+
+    [[nodiscard]] QString colorScheme() const
+    {
+        return mColorScheme;
+    }
+
 Q_SIGNALS:
 
     void rootPathChanged(const QStringList &rootPath);
@@ -95,12 +125,20 @@ Q_SIGNALS:
 
     void embeddedViewChanged();
 
+    void initialViewIndexChanged();
+
+    void playAtStartupChanged();
+
+    void colorSchemeChanged();
+
 public Q_SLOTS:
 
     void setRootPath(const QStringList &rootPath);
 
     void save();
 
+    void cancel();
+
     void setShowProgressInTaskBar(bool showProgressInTaskBar);
 
     void setShowSystemTrayIcon(bool showSystemTrayIcon);
@@ -109,6 +147,12 @@ public Q_SLOTS:
 
     void setEmbeddedView(ElisaUtils::PlayListEntryType embeddedView);
 
+    void setInitialViewIndex(int initialViewIndex);
+
+    void setPlayAtStartup(bool playAtStartup);
+
+    void setColorScheme(const QString &scheme);
+
 private Q_SLOTS:
 
     void configChanged();
@@ -129,7 +173,13 @@ private:
 
     bool mForceUsageOfFastFileSearch = true;
 
+    bool mPlayAtStartup = false;
+
+    QString mColorScheme;
+
     ElisaUtils::PlayListEntryType mEmbeddedView = ElisaUtils::Unknown;
+
+    int mInitialViewIndex = 2;
 };
 
 #endif
