@@ -1,24 +1,17 @@
 /*
- * Copyright 2016 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2016 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "manageaudioplayer.h"
 
 #include "mediaplaylist.h"
 
+#include "elisa_settings.h"
+
+#include "playerLogging.h"
+
 #include <QTimer>
 #include <QDateTime>
 
@@ -98,6 +91,8 @@ QVariantMap ManageAudioPlayer::persistentState() const
     persistentStateValue[QStringLiteral("isPlaying")] = mPlayingState;
 
     persistentStateValue[QStringLiteral("playerPosition")] = mPlayerPosition;
+    persistentStateValue[QStringLiteral("playerDuration")] = mAudioDuration;
+
     if (mCurrentTrack.isValid()) {
         persistentStateValue[QStringLiteral("audioPlayerCurrentTitle")] = mCurrentTrack.data(mTitleRole);
         persistentStateValue[QStringLiteral("audioPlayerCurrentArtistName")] = mCurrentTrack.data(mArtistNameRole);
@@ -416,13 +411,14 @@ void ManageAudioPlayer::setPlayerPosition(qint64 playerPosition)
 void ManageAudioPlayer::setCurrentPlayingForRadios(const QString &title, const QString &nowPlaying)
 {
     if (mPlayListModel && mCurrentTrack.isValid()) {
-        Q_EMIT currentPlayingForRadiosChanged(title, MediaPlayList::TitleRole);
-        Q_EMIT currentPlayingForRadiosChanged(nowPlaying, MediaPlayList::ArtistRole);
+        Q_EMIT updateData(mCurrentTrack, title, MediaPlayList::TitleRole);
+        Q_EMIT updateData(mCurrentTrack, nowPlaying, MediaPlayList::ArtistRole);
     }
 }
 
 void ManageAudioPlayer::setPlayControlPosition(int playerPosition)
 {
+    qCDebug(orgKdeElisaPlayer()) << "ManageAudioPlayer::setPlayControlPosition" << playerPosition;
     Q_EMIT seek(playerPosition);
 }
 
@@ -443,6 +439,7 @@ void ManageAudioPlayer::setPersistentState(const QVariantMap &persistentStateVal
 
 void ManageAudioPlayer::playerSeek(int position)
 {
+    qCDebug(orgKdeElisaPlayer()) << "ManageAudioPlayer::playerSeek" << position;
     Q_EMIT seek(position);
 }
 
@@ -581,15 +578,21 @@ void ManageAudioPlayer::restorePreviousState()
     }
 
     auto isPlaying = mPersistentState.find(QStringLiteral("isPlaying"));
-    if (isPlaying != mPersistentState.end() && mPlayingState != isPlaying->toBool()) {
+    auto currentConfiguration = Elisa::ElisaConfiguration::self();
+    bool autoPlay = currentConfiguration->playAtStartup();
+    if (autoPlay && isPlaying != mPersistentState.end() && mPlayingState != isPlaying->toBool()) {
         mPlayingState = isPlaying->toBool();
     }
 
     auto playerPosition = mPersistentState.find(QStringLiteral("playerPosition"));
     if (playerPosition != mPersistentState.end()) {
-        mPlayerPosition = playerPosition->toLongLong();
+        setPlayerPosition(playerPosition->toLongLong());
         Q_EMIT seek(mPlayerPosition);
     }
+    auto playerDuration = mPersistentState.find(QStringLiteral("playerDuration"));
+    if (playerDuration != mPersistentState.end()) {
+        setAudioDuration(playerDuration->toInt());
+    }
 
     mPersistentState.clear();
 }
