@@ -1,18 +1,7 @@
 /*
- * Copyright 2018 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2018 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "trackmetadatamodel.h"
@@ -21,7 +10,9 @@
 
 #include <KI18n/KLocalizedString>
 
-#include <QtConcurrentRun>
+#include <QtConcurrent>
+
+#include <algorithm>
 
 TrackMetadataModel::TrackMetadataModel(QObject *parent)
     : QAbstractListModel(parent)
@@ -43,7 +34,7 @@ int TrackMetadataModel::rowCount(const QModelIndex &parent) const
         return 0;
     }
 
-    return mTrackData.count();
+    return mTrackKeys.count();
 }
 
 QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
@@ -97,97 +88,23 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
             }
             break;
         }
+        case DataTypes::DurationRole:
+        {
+            auto trackDuration = mTrackData.duration();
+            if (trackDuration.hour() == 0) {
+                result = trackDuration.toString(QStringLiteral("mm:ss"));
+            } else {
+                result = trackDuration.toString();
+            }
+            break;
+        }
         default:
             result = mTrackData[currentKey];
             break;
         }
         break;
     case ItemNameRole:
-        switch (currentKey)
-        {
-        case DataTypes::TitleRole:
-            result = i18nc("Track title for track metadata view", "Title");
-            break;
-        case DataTypes::DurationRole:
-            result = i18nc("Duration label for track metadata view", "Duration");
-            break;
-        case DataTypes::ArtistRole:
-            result = i18nc("Track artist for track metadata view", "Artist");
-            break;
-        case DataTypes::AlbumRole:
-            result = i18nc("Album name for track metadata view", "Album");
-            break;
-        case DataTypes::AlbumArtistRole:
-            result = i18nc("Album artist for track metadata view", "Album Artist");
-            break;
-        case DataTypes::TrackNumberRole:
-            result = i18nc("Track number for track metadata view", "Track Number");
-            break;
-        case DataTypes::DiscNumberRole:
-            result = i18nc("Disc number for track metadata view", "Disc Number");
-            break;
-        case DataTypes::RatingRole:
-            result = i18nc("Rating label for information panel", "Rating");
-            break;
-        case DataTypes::GenreRole:
-            result = i18nc("Genre label for track metadata view", "Genre");
-            break;
-        case DataTypes::LyricistRole:
-            result = i18nc("Lyricist label for track metadata view", "Lyricist");
-            break;
-        case DataTypes::ComposerRole:
-            result = i18nc("Composer name for track metadata view", "Composer");
-            break;
-        case DataTypes::CommentRole:
-            result = i18nc("Comment label for track metadata view", "Comment");
-            break;
-        case DataTypes::YearRole:
-            result = i18nc("Year label for track metadata view", "Year");
-            break;
-        case DataTypes::ChannelsRole:
-            result = i18nc("Channels label for track metadata view", "Channels");
-            break;
-        case DataTypes::BitRateRole:
-            result = i18nc("Bit rate label for track metadata view", "Bit Rate");
-            break;
-        case DataTypes::SampleRateRole:
-            result = i18nc("Sample Rate label for track metadata view", "Sample Rate");
-            break;
-        case DataTypes::LastPlayDate:
-            result = i18nc("Last play date label for track metadata view", "Last played");
-            break;
-        case DataTypes::PlayCounter:
-            result = i18nc("Play counter label for track metadata view", "Play count");
-            break;
-        case DataTypes::LyricsRole:
-            result = i18nc("Lyrics label for track metadata view", "Lyrics");
-            break;
-        case DataTypes::ResourceRole:
-            result = i18nc("Radio HTTP address for radio metadata view", "Stream Http Address");
-            break;
-        case DataTypes::SecondaryTextRole:
-        case DataTypes::ImageUrlRole:
-        case DataTypes::ShadowForImageRole:
-        case DataTypes::ChildModelRole:
-        case DataTypes::StringDurationRole:
-        case DataTypes::IsValidAlbumArtistRole:
-        case DataTypes::AllArtistsRole:
-        case DataTypes::HighestTrackRating:
-        case DataTypes::IdRole:
-        case DataTypes::ParentIdRole:
-        case DataTypes::DatabaseIdRole:
-        case DataTypes::IsSingleDiscAlbumRole:
-        case DataTypes::ContainerDataRole:
-        case DataTypes::IsPartialDataRole:
-        case DataTypes::AlbumIdRole:
-        case DataTypes::HasEmbeddedCover:
-        case DataTypes::FileModificationTime:
-        case DataTypes::FirstPlayDate:
-        case DataTypes::PlayFrequency:
-        case DataTypes::ElementTypeRole:
-        case DataTypes::FullDataRole:
-            break;
-        }
+        result = nameFromRole(currentKey);
         break;
     case ItemTypeRole:
         switch (currentKey)
@@ -196,7 +113,8 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
             result = TextEntry;
             break;
         case DataTypes::ResourceRole:
-            result = TextEntry;
+        case DataTypes::ImageUrlRole:
+            result = UrlEntry;
             break;
         case DataTypes::ArtistRole:
             result = TextEntry;
@@ -240,12 +158,22 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
         case DataTypes::LyricsRole:
             result = LongTextEntry;
             break;
-        case DataTypes::DurationRole:
         case DataTypes::SampleRateRole:
+            result = IntegerEntry;
+            break;
         case DataTypes::BitRateRole:
+            result = IntegerEntry;
+            break;
         case DataTypes::ChannelsRole:
+            result = IntegerEntry;
+            break;
+        case DataTypes::FirstPlayDate:
+            result = DateEntry;
+            break;
+        case DataTypes::DurationRole:
+            result = DurationEntry;
+            break;
         case DataTypes::SecondaryTextRole:
-        case DataTypes::ImageUrlRole:
         case DataTypes::ShadowForImageRole:
         case DataTypes::ChildModelRole:
         case DataTypes::StringDurationRole:
@@ -261,10 +189,12 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
         case DataTypes::AlbumIdRole:
         case DataTypes::HasEmbeddedCover:
         case DataTypes::FileModificationTime:
-        case DataTypes::FirstPlayDate:
         case DataTypes::PlayFrequency:
         case DataTypes::ElementTypeRole:
         case DataTypes::FullDataRole:
+        case DataTypes::IsDirectoryRole:
+        case DataTypes::IsPlayListRole:
+        case DataTypes::FilePathRole:
             break;
         }
         break;
@@ -276,9 +206,12 @@ QVariant TrackMetadataModel::data(const QModelIndex &index, int role) const
 bool TrackMetadataModel::setData(const QModelIndex &index, const QVariant &value, int role)
 {
     if (data(index, role) != value) {
-        mTrackData[mTrackKeys[index.row()]] = value;
+        auto dataType = mTrackKeys[index.row()];
 
-        emit dataChanged(index, index, QVector<int>() << role);
+        mTrackData[dataType] = value;
+        mFullData[dataType] = value;
+
+        Q_EMIT dataChanged(index, index, QVector<int>() << role);
         return true;
     }
     return false;
@@ -296,7 +229,11 @@ QHash<int, QByteArray> TrackMetadataModel::roleNames() const
 
 QString TrackMetadataModel::fileUrl() const
 {
-    return mFileUrl;
+    if (mFileUrl.isLocalFile()) {
+        return mFileUrl.toLocalFile();
+    } else {
+        return mFileUrl.toString();
+    }
 }
 
 QUrl TrackMetadataModel::coverUrl() const
@@ -325,17 +262,22 @@ qulonglong TrackMetadataModel::databaseId() const
 
 void TrackMetadataModel::trackData(const TrackMetadataModel::TrackDataType &trackData)
 {
-    if (!mFullData.isEmpty() && trackData.databaseId() != mFullData.databaseId()) {
+    if ((mDatabaseId != 0 && trackData.databaseId() != mDatabaseId) ||
+            (!mFileUrl.isEmpty() && trackData.resourceURI() != mFileUrl) ||
+            (!mFullData.isEmpty() && trackData.databaseId() != mFullData.databaseId())) {
         return;
     }
 
     const QList<DataTypes::ColumnsRoles> fieldsForTrack({DataTypes::TitleRole, DataTypes::ArtistRole,
-                                                                 DataTypes::AlbumRole, DataTypes::AlbumArtistRole,
-                                                                 DataTypes::TrackNumberRole, DataTypes::DiscNumberRole,
-                                                                 DataTypes::RatingRole, DataTypes::GenreRole,
-                                                                 DataTypes::LyricistRole, DataTypes::ComposerRole,
-                                                                 DataTypes::CommentRole, DataTypes::YearRole,
-                                                                 DataTypes::LastPlayDate, DataTypes::PlayCounter});
+                                                         DataTypes::AlbumRole, DataTypes::AlbumArtistRole,
+                                                         DataTypes::TrackNumberRole, DataTypes::DiscNumberRole,
+                                                         DataTypes::RatingRole, DataTypes::GenreRole,
+                                                         DataTypes::LyricistRole, DataTypes::ComposerRole,
+                                                         DataTypes::CommentRole, DataTypes::YearRole,
+                                                         DataTypes::ChannelsRole, DataTypes::BitRateRole,
+                                                         DataTypes::SampleRateRole, DataTypes::LyricsRole,
+                                                         DataTypes::LastPlayDate, DataTypes::PlayCounter,
+                                                         DataTypes::DurationRole});
 
     fillDataFromTrackData(trackData, fieldsForTrack);
 }
@@ -350,12 +292,6 @@ void TrackMetadataModel::fillDataFromTrackData(const TrackMetadataModel::TrackDa
 
     for (DataTypes::ColumnsRoles role : fieldsForTrack) {
         if (trackData.constFind(role) != trackData.constEnd()) {
-            if (role == DataTypes::RatingRole) {
-                if (trackData[role].toInt() == 0) {
-                    continue;
-                }
-            }
-
             mTrackKeys.push_back(role);
             mTrackData[role] = trackData[role];
         }
@@ -373,11 +309,7 @@ void TrackMetadataModel::fillDataFromTrackData(const TrackMetadataModel::TrackDa
 
     auto rawFileUrl = trackData[DataTypes::ResourceRole].toUrl();
 
-    if (rawFileUrl.isLocalFile()) {
-        mFileUrl = rawFileUrl.toLocalFile();
-    } else {
-        mFileUrl = rawFileUrl.toString();
-    }
+    mFileUrl = rawFileUrl;
     Q_EMIT fileUrlChanged();
 }
 
@@ -409,6 +341,11 @@ void TrackMetadataModel::fillLyricsDataFromTrack()
     endInsertRows();
 }
 
+const TrackMetadataModel::TrackDataType &TrackMetadataModel::allTrackData() const
+{
+    return mFullData;
+}
+
 void TrackMetadataModel::lyricsValueIsReady()
 {
     if (!mLyricsValueWatcher.result().isEmpty()) {
@@ -422,12 +359,19 @@ void TrackMetadataModel::lyricsValueIsReady()
 
 void TrackMetadataModel::initializeByIdAndUrl(ElisaUtils::PlayListEntryType type, qulonglong databaseId, const QUrl &url)
 {
-    mFullData.clear();
-    mTrackData.clear();
-    mCoverImage.clear();
-    mFileUrl.clear();
+    if (!mFullData.isEmpty()) {
+        beginResetModel();
+        mFullData.clear();
+        mTrackData.clear();
+        mCoverImage.clear();
+        mFileUrl.clear();
+        endResetModel();
 
-    Q_EMIT lyricsChanged();
+        Q_EMIT lyricsChanged();
+    }
+
+    mFileUrl = url;
+    mDatabaseId = databaseId;
 
     Q_EMIT needDataByDatabaseIdAndUrl(type, databaseId, url);
 }
@@ -439,10 +383,6 @@ void TrackMetadataModel::initialize(MusicListenersManager *newManager, DatabaseI
 
     if (mManager) {
         mDataLoader.setDatabase(mManager->viewDatabase());
-        connect(this, &TrackMetadataModel::needDataByUrl,
-                mManager->tracksListener(), &TracksListener::trackByFileNameInList);
-        connect(mManager->tracksListener(), &TracksListener::trackHasChanged,
-                this, &TrackMetadataModel::trackData);
     } else if (trackDatabase) {
         mDataLoader.setDatabase(trackDatabase);
     }
@@ -455,10 +395,6 @@ void TrackMetadataModel::initialize(MusicListenersManager *newManager, DatabaseI
             &mDataLoader, &ModelDataLoader::loadDataByDatabaseIdAndUrl);
     connect(this, &TrackMetadataModel::needDataByUrl,
             &mDataLoader, &ModelDataLoader::loadDataByUrl);
-    connect(this, &TrackMetadataModel::saveRadioData,
-            &mDataLoader, &ModelDataLoader::saveRadioModified);
-    connect(this, &TrackMetadataModel::deleteRadioData,
-            &mDataLoader, &ModelDataLoader::removeRadio);
     connect(&mDataLoader, &ModelDataLoader::trackModified,
             this, &TrackMetadataModel::trackData);
     connect(&mDataLoader, &ModelDataLoader::allTrackData,
@@ -471,10 +407,184 @@ void TrackMetadataModel::initialize(MusicListenersManager *newManager, DatabaseI
             this, &TrackMetadataModel::radioData);
 }
 
+ModelDataLoader &TrackMetadataModel::modelDataLoader()
+{
+    return mDataLoader;
+}
+
+const TrackMetadataModel::TrackDataType &TrackMetadataModel::displayedTrackData() const
+{
+    return mTrackData;
+}
+
+DataTypes::ColumnsRoles TrackMetadataModel::trackKey(int index) const
+{
+    return mTrackKeys[index];
+}
+
+void TrackMetadataModel::removeDataByIndex(int index)
+{
+    auto dataKey = mTrackKeys[index];
+
+    mTrackData[dataKey] = {};
+    mFullData[dataKey] = {};
+    mTrackKeys.removeAt(index);
+}
+
+void TrackMetadataModel::addDataByName(const QString &name)
+{
+    DataTypes::ColumnsRoles newRole = DataTypes::TitleRole;
+
+    if (name == i18nc("Track title for track metadata view", "Title")) {
+        newRole = DataTypes::TitleRole;
+    } else if (name == i18nc("Track artist for track metadata view", "Artist")) {
+        newRole = DataTypes::ArtistRole;
+    } else if (name == i18nc("Album name for track metadata view", "Album")) {
+        newRole = DataTypes::AlbumRole;
+    } else if (name == i18nc("Album artist for track metadata view", "Album Artist")) {
+        newRole = DataTypes::AlbumArtistRole;
+    } else if (name == i18nc("Track number for track metadata view", "Track Number")) {
+        newRole = DataTypes::TrackNumberRole;
+    } else if (name == i18nc("Disc number for track metadata view", "Disc Number")) {
+        newRole = DataTypes::DiscNumberRole;
+    } else if (name == i18nc("Rating label for information panel", "Rating")) {
+        newRole = DataTypes::RatingRole;
+    } else if (name == i18nc("Genre label for track metadata view", "Genre")) {
+        newRole = DataTypes::GenreRole;
+    } else if (name == i18nc("Lyricist label for track metadata view", "Lyricist")) {
+        newRole = DataTypes::LyricistRole;
+    } else if (name == i18nc("Composer name for track metadata view", "Composer")) {
+        newRole = DataTypes::ComposerRole;
+    } else if (name == i18nc("Comment label for track metadata view", "Comment")) {
+        newRole = DataTypes::CommentRole;
+    } else if (name == i18nc("Year label for track metadata view", "Year")) {
+        newRole = DataTypes::YearRole;
+    } else if (name == i18nc("Channels label for track metadata view", "Channels")) {
+        newRole = DataTypes::ChannelsRole;
+    } else if (name == i18nc("Bit rate label for track metadata view", "Bit Rate")) {
+        newRole = DataTypes::BitRateRole;
+    } else if (name == i18nc("Sample Rate label for track metadata view", "Sample Rate")) {
+        newRole = DataTypes::SampleRateRole;
+    } else if (name == i18nc("Lyrics label for track metadata view", "Lyrics")) {
+        newRole = DataTypes::LyricsRole;
+    } else if (name == i18nc("Duration label for track metadata view", "Duration")) {
+        newRole = DataTypes::DurationRole;
+    }
+
+    mTrackData[newRole] = {};
+    mFullData[newRole] = {};
+    mTrackKeys.push_back(newRole);
+}
+
+QString TrackMetadataModel::nameFromRole(DataTypes::ColumnsRoles role)
+{
+    auto result = QString{};
+    switch (role)
+    {
+    case DataTypes::TitleRole:
+        result = i18nc("Track title for track metadata view", "Title");
+        break;
+    case DataTypes::DurationRole:
+        result = i18nc("Duration label for track metadata view", "Duration");
+        break;
+    case DataTypes::ArtistRole:
+        result = i18nc("Track artist for track metadata view", "Artist");
+        break;
+    case DataTypes::AlbumRole:
+        result = i18nc("Album name for track metadata view", "Album");
+        break;
+    case DataTypes::AlbumArtistRole:
+        result = i18nc("Album artist for track metadata view", "Album Artist");
+        break;
+    case DataTypes::TrackNumberRole:
+        result = i18nc("Track number for track metadata view", "Track Number");
+        break;
+    case DataTypes::DiscNumberRole:
+        result = i18nc("Disc number for track metadata view", "Disc Number");
+        break;
+    case DataTypes::RatingRole:
+        result = i18nc("Rating label for information panel", "Rating");
+        break;
+    case DataTypes::GenreRole:
+        result = i18nc("Genre label for track metadata view", "Genre");
+        break;
+    case DataTypes::LyricistRole:
+        result = i18nc("Lyricist label for track metadata view", "Lyricist");
+        break;
+    case DataTypes::ComposerRole:
+        result = i18nc("Composer name for track metadata view", "Composer");
+        break;
+    case DataTypes::CommentRole:
+        result = i18nc("Comment label for track metadata view", "Comment");
+        break;
+    case DataTypes::YearRole:
+        result = i18nc("Year label for track metadata view", "Year");
+        break;
+    case DataTypes::ChannelsRole:
+        result = i18nc("Channels label for track metadata view", "Channels");
+        break;
+    case DataTypes::BitRateRole:
+        result = i18nc("Bit rate label for track metadata view", "Bit Rate");
+        break;
+    case DataTypes::SampleRateRole:
+        result = i18nc("Sample Rate label for track metadata view", "Sample Rate");
+        break;
+    case DataTypes::LastPlayDate:
+        result = i18nc("Last play date label for track metadata view", "Last played");
+        break;
+    case DataTypes::FirstPlayDate:
+        result = i18nc("First play date label for track metadata view", "First played");
+        break;
+    case DataTypes::PlayCounter:
+        result = i18nc("Play counter label for track metadata view", "Play count");
+        break;
+    case DataTypes::LyricsRole:
+        result = i18nc("Lyrics label for track metadata view", "Lyrics");
+        break;
+    case DataTypes::ResourceRole:
+        result = i18nc("Radio HTTP address for radio metadata view", "Stream Http Address");
+        break;
+    case DataTypes::ImageUrlRole:
+        result = i18nc("Image address for radio metadata view", "Image Address");
+        break;
+    case DataTypes::SecondaryTextRole:
+    case DataTypes::ShadowForImageRole:
+    case DataTypes::ChildModelRole:
+    case DataTypes::StringDurationRole:
+    case DataTypes::IsValidAlbumArtistRole:
+    case DataTypes::AllArtistsRole:
+    case DataTypes::HighestTrackRating:
+    case DataTypes::IdRole:
+    case DataTypes::ParentIdRole:
+    case DataTypes::DatabaseIdRole:
+    case DataTypes::IsSingleDiscAlbumRole:
+    case DataTypes::ContainerDataRole:
+    case DataTypes::IsPartialDataRole:
+    case DataTypes::AlbumIdRole:
+    case DataTypes::HasEmbeddedCover:
+    case DataTypes::FileModificationTime:
+    case DataTypes::PlayFrequency:
+    case DataTypes::ElementTypeRole:
+    case DataTypes::FullDataRole:
+    case DataTypes::IsDirectoryRole:
+    case DataTypes::IsPlayListRole:
+    case DataTypes::FilePathRole:
+        break;
+    }
+    return result;
+}
+
+bool TrackMetadataModel::metadataExists(DataTypes::ColumnsRoles metadataRole) const
+{
+    return std::find(mTrackKeys.begin(), mTrackKeys.end(), metadataRole) != mTrackKeys.end();
+}
+
 void TrackMetadataModel::fetchLyrics()
 {
-    auto lyricicsValue = QtConcurrent::run(QThreadPool::globalInstance(), [=]() {
-        auto trackData = mFileScanner.scanOneFile(mFullData[DataTypes::ResourceRole].toUrl());
+    auto fileUrl = mFullData[DataTypes::ResourceRole].toUrl();
+    auto lyricicsValue = QtConcurrent::run(QThreadPool::globalInstance(), [fileUrl, this]() {
+        auto locker = QMutexLocker(&mFileScannerMutex);
+        auto trackData = mFileScanner.scanOneFile(fileUrl);
         if (!trackData.lyrics().isEmpty()) {
             return trackData.lyrics();
         }
@@ -495,24 +605,23 @@ void TrackMetadataModel::initializeForNewRadio()
 void TrackMetadataModel::fillDataForNewRadio()
 {
     beginResetModel();
+    mFullData.clear();
     mTrackData.clear();
     mTrackKeys.clear();
 
-    for (auto role : {
-         DataTypes::TitleRole,
-         DataTypes::ResourceRole,
-         DataTypes::CommentRole,
-         DataTypes::DatabaseIdRole
+    auto allRoles = {DataTypes::TitleRole, DataTypes::ResourceRole,
+                     DataTypes::CommentRole, DataTypes::ImageUrlRole};
 
-}) {
+    for (auto role : allRoles) {
         mTrackKeys.push_back(role);
         if (role == DataTypes::DatabaseIdRole) {
-            mTrackData[role] = -1;
+            mFullData[role] = -1;
         } else {
-            mTrackData[role] = QString();
+            mFullData[role] = QString();
         }
-
     }
+    mTrackData = mFullData;
+    mFullData[DataTypes::ElementTypeRole] = ElisaUtils::Radio;
     filterDataFromTrackData();
     endResetModel();
 }
@@ -526,6 +635,8 @@ void TrackMetadataModel::initializeByUrl(ElisaUtils::PlayListEntryType type, con
 
     Q_EMIT lyricsChanged();
 
+    mFileUrl = url;
+
     Q_EMIT needDataByUrl(type, url);
 }
 
@@ -539,18 +650,6 @@ void TrackMetadataModel::setDatabase(DatabaseInterface *trackDatabase)
     initialize(nullptr, trackDatabase);
 }
 
-void TrackMetadataModel::saveData()
-{
-    Q_EMIT saveRadioData(mTrackData);
-}
-
-void TrackMetadataModel::deleteRadio()
-{
-    if (mTrackData[DataTypes::DatabaseIdRole]>=0) {
-        Q_EMIT deleteRadioData(mTrackData[DataTypes::DatabaseIdRole].toULongLong());
-    }
-}
-
 void TrackMetadataModel::radioData(const TrackDataType &radiosData)
 {
     if (!mFullData.isEmpty() && mFullData[DataTypes::DatabaseIdRole].toInt() != -1 &&
@@ -559,7 +658,7 @@ void TrackMetadataModel::radioData(const TrackDataType &radiosData)
     }
 
     const QList<DataTypes::ColumnsRoles> fieldsForTrack({DataTypes::TitleRole, DataTypes::ResourceRole,
-                                                                 DataTypes::CommentRole, DataTypes::DatabaseIdRole});
+                                                                 DataTypes::CommentRole, DataTypes::ImageUrlRole});
 
     fillDataFromTrackData(radiosData, fieldsForTrack);
 }
