@@ -1,18 +1,7 @@
 /*
- * Copyright 2016-2017 Matthieu Gallien <matthieu_gallien@yahoo.fr>
- *
- * This program is free software: you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public
- * License as published by the Free Software Foundation; either
- * version 3 of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Lesser General Public License for more details.
- *
- * You should have received a copy of the GNU Lesser General Public License
- * along with this program.  If not, see <https://www.gnu.org/licenses/>.
+   SPDX-FileCopyrightText: 2016 (c) Matthieu Gallien <matthieu_gallien@yahoo.fr>
+
+   SPDX-License-Identifier: LGPL-3.0-or-later
  */
 
 #include "musiclistenersmanager.h"
@@ -46,18 +35,12 @@
 #include <KI18n/KLocalizedString>
 
 #include <QThread>
-#include <QMutex>
 #include <QStandardPaths>
 #include <QDir>
 #include <QCoreApplication>
-#include <QList>
-#include <QScopedPointer>
-#include <QPointer>
 #include <QFileSystemWatcher>
 #include <QAction>
 
-#include <QDebug>
-
 #include <list>
 
 class MusicListenersManagerPrivate
@@ -90,6 +73,8 @@ public:
 
     QFileSystemWatcher mConfigFileWatcher;
 
+    QStringList mPreviousRootPathValue;
+
     ElisaApplication *mElisaApplication = nullptr;
 
     int mImportedTracksCount = 0;
@@ -154,7 +139,13 @@ MusicListenersManager::MusicListenersManager(QObject *parent)
 }
 
 MusicListenersManager::~MusicListenersManager()
-= default;
+{
+    d->mListenerThread.quit();
+    d->mListenerThread.wait();
+
+    d->mDatabaseThread.quit();
+    d->mDatabaseThread.wait();
+}
 
 DatabaseInterface *MusicListenersManager::viewDatabase() const
 {
@@ -304,6 +295,7 @@ void MusicListenersManager::deleteElementById(ElisaUtils::PlayListEntryType entr
     case ElisaUtils::Track:
     case ElisaUtils::FileName:
     case ElisaUtils::Unknown:
+    case ElisaUtils::Container:
         break;
     }
 }
@@ -327,10 +319,28 @@ void MusicListenersManager::configChanged()
     currentConfiguration->load();
     currentConfiguration->read();
 
+    bool configurationHasChanged = false;
+#if defined KF5Baloo_FOUND && KF5Baloo_FOUND
+    if (d->mBalooIndexerAvailable && d->mBalooIndexerActive && d->mBalooListener.canHandleRootPaths() && !currentConfiguration->forceUsageOfFastFileSearch()) {
+        configurationHasChanged = true;
+    } else if (d->mBalooIndexerAvailable && !d->mBalooIndexerActive && d->mBalooListener.canHandleRootPaths() && currentConfiguration->forceUsageOfFastFileSearch()) {
+        configurationHasChanged = true;
+    }
+#endif
+
+    auto inputRootPath = currentConfiguration->rootPath();
+    configurationHasChanged = configurationHasChanged || (d->mPreviousRootPathValue != inputRootPath);
+
+    if (configurationHasChanged) {
+        d->mPreviousRootPathValue = inputRootPath;
+    } else {
+        qCDebug(orgKdeElisaIndexersManager()) << "music paths configuration and scanning has not changed";
+        return;
+    }
+
     //resolve symlinks
     QStringList allRootPaths;
-    auto inputRootPath = currentConfiguration->rootPath();
-    for (const auto &onePath : inputRootPath) {
+    for (const auto &onePath : qAsConst(inputRootPath)) {
         auto workPath = onePath;
         if (workPath.startsWith(QLatin1String("file:/"))) {
             auto urlPath = QUrl{workPath};
@@ -363,16 +373,17 @@ void MusicListenersManager::configChanged()
     }
 
 #if defined KF5Baloo_FOUND && KF5Baloo_FOUND
-    if (d->mBalooIndexerAvailable && !d->mBalooIndexerActive && d->mBalooListener.canHandleRootPaths())
-    {
+    if (d->mBalooIndexerAvailable && !d->mBalooIndexerActive && d->mBalooListener.canHandleRootPaths() && currentConfiguration->forceUsageOfFastFileSearch()) {
         qCDebug(orgKdeElisaIndexersManager()) << "trigger start of baloo file indexer";
         QMetaObject::invokeMethod(d->mFileListener.fileListing(), "stop", Qt::BlockingQueuedConnection);
         d->mFileSystemIndexerActive = false;
         startBalooIndexing();
-    } else if (!d->mFileSystemIndexerActive && d->mBalooIndexerActive && !d->mBalooListener.canHandleRootPaths())
-    {
-        qCDebug(orgKdeElisaIndexersManager()) << "trigger stop of baloo file indexer";
-        QMetaObject::invokeMethod(d->mBalooListener.fileListing(), "stop", Qt::BlockingQueuedConnection);
+    } else if ((!d->mFileSystemIndexerActive && d->mBalooIndexerActive && !d->mBalooListener.canHandleRootPaths()) ||
+               !currentConfiguration->forceUsageOfFastFileSearch()) {
+        if (d->mBalooIndexerActive) {
+            qCDebug(orgKdeElisaIndexersManager()) << "trigger stop of baloo file indexer";
+            QMetaObject::invokeMethod(d->mBalooListener.fileListing(), "stop", Qt::BlockingQueuedConnection);
+        }
         d->mBalooIndexerActive = false;
         startLocalFileSystemIndexing();
     }
